/*
 * fwflash.c - handling the betty flashes
 *
 * author: hackbard@hackdaworld.org
 *
 */

/*
 * include files
 */

#include <stdio.h>

#include "lpc2xxx.h"

/*
 * defines
 */

/* bank 0/2 addr */
#define BANK0			0x80000000
#define BANK2			0x82000000

/* flash cmd addresses - flash[0-18] <--> arm[1-19]*/
#define B0F555	(*((volatile unsigned long *)(BANK0|0xaaa)))	// 0x555
#define B0F2AA	(*((volatile unsigned long *)(BANK0|0x554)))	// 0x2aa
#define B2F555	(*((volatile unsigned long *)(BANK2|0xaaa)))	// 0x555
#define B2F2AA	(*((volatile unsigned long *)(BANK2|0x554)))	// 0x2aa

/* commands */
#define CMD_READ		'R'
#define CMD_CHIP_ERASE		'E'

#define BUFSIZE			16

/*
 * type definitions
 */

typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;

/*
 * define macros
 */

#define TX_BYTE(x)		while(!(UART0_LSR&(1<<5))) continue; \
				UART0_THR=x

#define TX_NL			TX_BYTE('\n'); TX_BYTE('\r')

#define TX_STRING(x,len)	for(scnt=0;scnt<len;scnt++) { 	\
					TX_BYTE(x[scnt]);	\
				}				\
				TX_NL

/*
 * function prototypes
 */

void mmap_init(u8 memtype);
void uart0_init(void);
void uart0_send_string(char *txbuf);
void uart0_send_char(char send);

/*
 * global variables
 */

int scnt;

/*
 * main function
 */

int main(void) {

	/* variables */

	u32 i,addrlen,datalen;
	u8 buf[BUFSIZE];
	u32 addr;
	u16 *dptr;
	u8 cmd;
	u8 txrx;

	/* memory mapping of interrupt vectors to static ram */

	//mmap_init(MMAP_RAM);

	/* uart initialization */

	//uart0_init();

	PINSEL0=0x05;			// pin select -> tx, rx
	UART0_FCR=0x07;			// enable fifo
	UART0_LCR=0x83;			// set dlab + word length
	UART0_DLL=0x10;			// br: 9600 @ 10/4 mhz
	UART0_DLM=0x00;
	UART0_LCR=0x03;			// unset dlab

	/* external memory init */

	BCFG0=0x1000FBEF;		// BCFG2 should be fine as is

	/* begin the main loop */
	while(1) {

	/* receive cmd */
	while(1) {

		while(!(UART0_LSR&(1<<0)))
			continue;
		cmd=UART0_RBR;

		if(cmd==CMD_CHIP_ERASE) {
			addrlen=0;
			datalen=1;
			break;
		}

		if(cmd==CMD_READ) {
			addrlen=4;
			datalen=1;
			break;
		}
	}

	/* receive (only if there is) more data from uart0 */

	addr=0;
	for(i=0;i<addrlen;i++) {
		//TX_STRING("got addr",8);
		while(!(UART0_LSR&(1<<0)))
			continue;
		txrx=UART0_RBR;
		//addr|=(txrx<<((addrlen-i-1)*4));
		addr=0x80000000;
	}

	for(i=0;i<datalen;i++) {
		//TX_STRING("got data",8);
		while(!(UART0_LSR&(1<<0)))
			continue;
		buf[i]=UART0_RBR;
	}

	/* process the cmd */

	switch(cmd) {
		case CMD_READ:
			dptr=(u16 *)addr;
			for(i=0;i<buf[0];i++) {
				TX_BYTE(*dptr);
				dptr++;
			}
			break;	
		case CMD_CHIP_ERASE:
			if(buf[0]=='0') {
				B0F555=0xaa;
				B0F2AA=0x55;
				B0F555=0x80;
				B0F555=0xaa;
				B0F2AA=0x55;
				B0F555=0x10;
			}
			else if(buf[0]=='2') {
				B2F555=0xaa;
				B2F2AA=0x55;
				B2F555=0x80;
				B2F555=0xaa;
				B2F2AA=0x55;
				B2F555=0x10;
			}
			break;
		default:
			break;
	}

	/* send an ack, the cmd! */
	TX_BYTE(cmd);
		
	}

	return 0;
}

/*
 * functions
 */

void mmap_init(u8 memtype) {

	MEMMAP=memtype;
}

void uart0_init(void) {

	PINSEL0=0x05;			// pin select -> tx, rx
	UART0_FCR=0x07;			// enable fifo
	UART0_LCR=0x83;			// set dlab + word length
	UART0_DLL=0x10;			// br: 9600 @ 10/4 mhz
	UART0_DLM=0x00;
	UART0_LCR=0x03;			// unset dlab
}

void uart0_send_string(char *txbuf) {

	int i;

	i=0;

	while(txbuf[i])
		UART0_THR=txbuf[i++];
	UART0_THR='\n';
	UART0_THR='\r';

	while(!(UART0_LSR&(1<<6))) {}
}

void uart0_send_char(char send) {

	while(!(UART0_LSR&(1<<5)))
		continue;

	UART0_THR=send;
}

